package com.nisum.webfluxreactivemongodb.controller;

import com.nisum.webfluxreactivemongodb.dto.ProductDto;
import com.nisum.webfluxreactivemongodb.service.ProductService;
import lombok.AllArgsConstructor;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("api/products")
@AllArgsConstructor
public class ProductController {
    @Autowired
    private ProductService productService;

@PostMapping
    public Mono<ProductDto> saveproduct(@RequestBody Mono<ProductDto> productDto){
        return productService.saveProduct(productDto);
    }
    @GetMapping("{id}")
    public Mono<ProductDto>getProductById(@PathVariable("id")int id){
    return productService.getProduct(id);
    }

    @GetMapping
    public Flux<ProductDto>getAllProducts(){
    return productService.getAllProducts();
    }

    @PutMapping("{id}")
    public Mono<ProductDto> updateProduct(@RequestBody Mono<ProductDto> productDto,@PathVariable("id")int id){
    return productService.updateProduct(productDto,id);
    }

    @DeleteMapping("{id}")
    public Mono<Void> deleteProduct(@PathVariable("id") int id) {
        return productService.deleteProduct(id);
    }
    @GetMapping("/product-range")
    public Flux<ProductDto> getProductBetweenRange(@RequestParam("min") double min, @RequestParam("max")double max){
        return productService.getProductInRange(min,max);
    }

    @GetMapping("/search")
    public Mono<ProductDto> getSearchAllProducts( @PathVariable(name="name") String name,@PathVariable(name="quantity") int quantity, @PathVariable(name="price") Double price) {
        if(null != name || !Strings.isBlank(name)){
            return productService.getProductByName(name);
        }else if(quantity>0){
            return productService.getProductByQuantity(quantity);
        }
        return productService.getProductByPrice(price);
    }



}
