package com.nisum.webfluxreactivemongodb.service;

import com.nisum.webfluxreactivemongodb.dto.ProductDto;
import com.nisum.webfluxreactivemongodb.entity.Product;
import com.nisum.webfluxreactivemongodb.mapper.ProductMapper;
import com.nisum.webfluxreactivemongodb.repository.ProductRepository;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Range;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
@Service
@AllArgsConstructor
public abstract class ProductServiceImpl implements ProductService {
    @Autowired
private ProductRepository productRepository;
   // @Override
    public Mono<ProductDto> saveProduct(ProductDto productDto) {
        Product product = ProductMapper.mapToProduct(productDto);
        Mono<Product> savedProduct= productRepository.save(product);
        return savedProduct
                .map((product1) -> ProductMapper.mapToProductDto(product1));

    }

    @Override
    public Mono<ProductDto> getProduct(int id) {
        Mono<Product> productMono = productRepository.findById(id);
        return productMono.map((product -> ProductMapper.mapToProductDto(product)));
    }

    @Override
    public Flux<ProductDto> getAllProducts() {
        Flux<Product> productFlux = productRepository.findAll();
        return productFlux.
                map(product -> ProductMapper.mapToProductDto(product))
                .switchIfEmpty(Flux.empty());
    }

   // @Override
    public Mono<ProductDto> updateProduct(ProductDto productDto, int id) {
      Mono<Product> productMono = productRepository.findById(id);
      return productMono.flatMap((existingProduct)->{
          existingProduct.setName(productDto.getName());
          existingProduct.setQuantity(productDto.getQuantity());
          existingProduct.setPrice(productDto.getPrice());
          return productRepository.save(existingProduct);
      }).map(product -> ProductMapper.mapToProductDto(product));
    }

    @Override
    public Mono<Void> deleteProduct(int id) {
       return productRepository.deleteById(id);
    }

    public Flux<ProductDto> getProductInRange(double min,double max)
    {
        return productRepository.findByPriceBetween(Range.closed(min,max));
    }
    @Override
    public Mono<ProductDto> getProductByName(String name) {
        Mono<Product> productMono = productRepository.findByName(name);
        return productMono.map((product -> ProductMapper.mapToProductDto(product)));
    }

    @Override
    public Mono<ProductDto> getProductByQuantity(int quantity) {
        Mono<Product> productMono = productRepository.findByQuantity(quantity);
        return productMono.map((product -> ProductMapper.mapToProductDto(product)));
    }

    @Override
    public Mono<ProductDto> getProductByPrice(double price) {
        Mono<Product> productMono = productRepository.findByPrice(price);
        return productMono.map((product -> ProductMapper.mapToProductDto(product)));
    }
}
